using System;
using System.Collections.Generic;
using DataContracts;
namespace Ato.EN.IntegrationServices.CodeGenerationDDCTNS
{
    public partial class DDCTNS2019Validator
    {
        #region Manual Rules - Rules marked for manual coding

        #region VR.ATO.DDCTNS.000151

        /*  VR.ATO.DDCTNS.000151
        Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"

        Legacy Rule Format:
        (^Context <> NULL AND ^IdentifierScheme <> 'http://www.ato.gov.au/tfn')

        Technical Business Rule Format:
        ^Context <> NULL AND ^IdentifierScheme <> 'http://www.ato.gov.au/tfn'

        Data Elements: 
        ^Context = Context
        ^IdentifierScheme = IdentifierScheme
        */
        protected void VRATODDCTNS000151(List<ProcessMessageDocument> response, DDCTNS2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = ((report.RPIdentifierScheme != null && (report.RPIdentifierScheme != "http://www.ato.gov.au/tfn")) ||
                (report.RPCarSeqNumContextCollection != null && report.RPCarSeqNumContextCollection.Find(context => context.IdentifierScheme != "http://www.ato.gov.au/tfn") != null) ||
                (report.RPEduSeqNumContextCollection != null && report.RPEduSeqNumContextCollection.Find(context => context.IdentifierScheme != "http://www.ato.gov.au/tfn") != null) ||
                (report.RPMangTaxAffSeqNumContextCollection != null && report.RPMangTaxAffSeqNumContextCollection.Find(context => context.IdentifierScheme != "http://www.ato.gov.au/tfn") != null) ||
                (report.RPSCSeqNumContextCollection != null && report.RPSCSeqNumContextCollection.Find(context => context.IdentifierScheme != "http://www.ato.gov.au/tfn") != null) ||
                (report.RPTrvlSeqNumContextCollection != null && report.RPTrvlSeqNumContextCollection.Find(context => context.IdentifierScheme != "http://www.ato.gov.au/tfn") != null) ||
                (report.RPProjPoolSeqNumContextCollection != null && report.RPProjPoolSeqNumContextCollection.Find(context => context.IdentifierScheme != "http://www.ato.gov.au/tfn") != null) ||
                (report.RPForestManInvSeqNumContextCollection != null && report.RPForestManInvSeqNumContextCollection.Find(context => context.IdentifierScheme != "http://www.ato.gov.au/tfn") != null) ||
                (report.RPGiftDonSeqNumContextCollection != null && report.RPGiftDonSeqNumContextCollection.Find(context => context.IdentifierScheme != "http://www.ato.gov.au/tfn") != null) ||
                (report.RPIntDedSeqNumContextCollection != null && report.RPIntDedSeqNumContextCollection.Find(context => context.IdentifierScheme != "http://www.ato.gov.au/tfn") != null) ||
                (report.RPDivDedSeqNumContextCollection != null && report.RPDivDedSeqNumContextCollection.Find(context => context.IdentifierScheme != "http://www.ato.gov.au/tfn") != null) ||
                (report.RPOTHWRSeqNumContextCollection != null && report.RPOTHWRSeqNumContextCollection.Find(context => context.IdentifierScheme != "http://www.ato.gov.au/tfn") != null) ||
                (report.RPClthngSeqNumContextCollection != null && report.RPClthngSeqNumContextCollection.Find(context => context.IdentifierScheme != "http://www.ato.gov.au/tfn") != null) ||
                (report.RPOthDedSeqNumContextCollection != null && report.RPOthDedSeqNumContextCollection.Find(context => context.IdentifierScheme != "http://www.ato.gov.au/tfn") != null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000151",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Entity identifier scheme must be \"http://www.ato.gov.au/tfn\" across all contexts",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000151" } },
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.DDCTNS.000151

        #region VR.ATO.DDCTNS.000152

        /*  VR.ATO.DDCTNS.000152
        TFN in entity identifier of context does not match the TFN provided for the Reporting Party

        Legacy Rule Format:
        (^Context <> NULL AND ^TFN <> ^DDCTNS302)

        Technical Business Rule Format:
        ^Context <> NULL AND ^TFN <> ^DDCTNS302

        Data Elements:
        ^Context = Context
        ^DDCTNS302 = RP:Identifiers.TaxFileNumber.Identifier
        ^TFN = TFN
        */
        protected void VRATODDCTNS000152(List<ProcessMessageDocument> response, DDCTNS2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = ((report.RPCarSeqNumContextCollection != null && report.RPCarSeqNumContextCollection.Find(context => context.IdentifierTFN != report.DDCTNS302) != null) ||
            (report.RPEduSeqNumContextCollection != null && report.RPEduSeqNumContextCollection.Find(context => context.IdentifierTFN != report.DDCTNS302) != null) ||
            (report.RPMangTaxAffSeqNumContextCollection != null && report.RPMangTaxAffSeqNumContextCollection.Find(context => context.IdentifierTFN != report.DDCTNS302) != null) ||
            (report.RPSCSeqNumContextCollection != null && report.RPSCSeqNumContextCollection.Find(context => context.IdentifierTFN != report.DDCTNS302) != null) ||
            (report.RPTrvlSeqNumContextCollection != null && report.RPTrvlSeqNumContextCollection.Find(context => context.IdentifierTFN != report.DDCTNS302) != null) ||
            (report.RPProjPoolSeqNumContextCollection != null && report.RPProjPoolSeqNumContextCollection.Find(context => context.IdentifierTFN != report.DDCTNS302) != null) ||
            (report.RPForestManInvSeqNumContextCollection != null && report.RPForestManInvSeqNumContextCollection.Find(context => context.IdentifierTFN != report.DDCTNS302) != null) ||
            (report.RPGiftDonSeqNumContextCollection != null && report.RPGiftDonSeqNumContextCollection.Find(context => context.IdentifierTFN != report.DDCTNS302) != null) ||
            (report.RPIntDedSeqNumContextCollection != null && report.RPIntDedSeqNumContextCollection.Find(context => context.IdentifierTFN != report.DDCTNS302) != null) ||
            (report.RPDivDedSeqNumContextCollection != null && report.RPDivDedSeqNumContextCollection.Find(context => context.IdentifierTFN != report.DDCTNS302) != null) ||
            (report.RPOTHWRSeqNumContextCollection != null && report.RPOTHWRSeqNumContextCollection.Find(context => context.IdentifierTFN != report.DDCTNS302) != null) ||
            (report.RPClthngSeqNumContextCollection != null && report.RPClthngSeqNumContextCollection.Find(context => context.IdentifierTFN != report.DDCTNS302) != null) ||
            (report.RPOthDedSeqNumContextCollection != null && report.RPOthDedSeqNumContextCollection.Find(context => context.IdentifierTFN != report.DDCTNS302) != null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000152" } },
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.DDCTNS.000152

        #region VR.ATO.DDCTNS.000153

        /*  VR.ATO.DDCTNS.000153
        Invalid context. Period start date must be the same across all duration contexts.

        Legacy Rule Format:
        (^Context <> NULL AND ^period.startDate <> ^DDCTNS300)

        Technical Business Rule Format:
        ^Context <> NULL AND ^period.startDate <> ^DDCTNS300

        Data Elements:
        ^Context = Context
        ^DDCTNS300 = RP:Lodgment.PeriodStart.Date
        ^period.startDate = period.startDate
        */
        protected void VRATODDCTNS000153(List<ProcessMessageDocument> response, DDCTNS2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = ((report.RPCarSeqNumContextCollection != null && report.RPCarSeqNumContextCollection.Find(context => context.StartDate != report.DDCTNS300) != null) ||
            (report.RPEduSeqNumContextCollection != null && report.RPEduSeqNumContextCollection.Find(context => context.StartDate != report.DDCTNS300) != null) ||
            (report.RPMangTaxAffSeqNumContextCollection != null && report.RPMangTaxAffSeqNumContextCollection.Find(context => context.StartDate != report.DDCTNS300) != null) ||
            (report.RPSCSeqNumContextCollection != null && report.RPSCSeqNumContextCollection.Find(context => context.StartDate != report.DDCTNS300) != null) ||
            (report.RPTrvlSeqNumContextCollection != null && report.RPTrvlSeqNumContextCollection.Find(context => context.StartDate != report.DDCTNS300) != null) ||
            (report.RPProjPoolSeqNumContextCollection != null && report.RPProjPoolSeqNumContextCollection.Find(context => context.StartDate != report.DDCTNS300) != null) ||
            (report.RPForestManInvSeqNumContextCollection != null && report.RPForestManInvSeqNumContextCollection.Find(context => context.StartDate != report.DDCTNS300) != null) ||
            (report.RPGiftDonSeqNumContextCollection != null && report.RPGiftDonSeqNumContextCollection.Find(context => context.StartDate != report.DDCTNS300) != null) ||
            (report.RPIntDedSeqNumContextCollection != null && report.RPIntDedSeqNumContextCollection.Find(context => context.StartDate != report.DDCTNS300) != null) ||
            (report.RPDivDedSeqNumContextCollection != null && report.RPDivDedSeqNumContextCollection.Find(context => context.StartDate != report.DDCTNS300) != null) ||
            (report.RPOTHWRSeqNumContextCollection != null && report.RPOTHWRSeqNumContextCollection.Find(context => context.StartDate != report.DDCTNS300) != null) ||
            (report.RPClthngSeqNumContextCollection != null && report.RPClthngSeqNumContextCollection.Find(context => context.StartDate != report.DDCTNS300) != null) ||
            (report.RPOthDedSeqNumContextCollection != null && report.RPOthDedSeqNumContextCollection.Find(context => context.StartDate != report.DDCTNS300) != null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500030",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period start date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000153" } },
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.DDCTNS.000153

        #region VR.ATO.DDCTNS.000154

        /*  VR.ATO.DDCTNS.000154
        Invalid context. Period end date must be the same across all duration contexts.

        Legacy Rule Format:
        (^Context <> NULL AND ^period.endDate <> ^DDCTNS301)

        Technical Business Rule Format:
        ^Context <> NULL AND ^period.endDate <> ^DDCTNS301

        Data Elements:
        ^Context = Context
        ^DDCTNS301 = RP:Lodgment.PeriodEnd.Date
        ^period.endDate = period.endDate
        */
        protected void VRATODDCTNS000154(List<ProcessMessageDocument> response, DDCTNS2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = ((report.RPCarSeqNumContextCollection != null && report.RPCarSeqNumContextCollection.Find(context => context.EndDate != report.DDCTNS301) != null) ||
            (report.RPEduSeqNumContextCollection != null && report.RPEduSeqNumContextCollection.Find(context => context.EndDate != report.DDCTNS301) != null) ||
            (report.RPMangTaxAffSeqNumContextCollection != null && report.RPMangTaxAffSeqNumContextCollection.Find(context => context.EndDate != report.DDCTNS301) != null) ||
            (report.RPSCSeqNumContextCollection != null && report.RPSCSeqNumContextCollection.Find(context => context.EndDate != report.DDCTNS301) != null) ||
            (report.RPTrvlSeqNumContextCollection != null && report.RPTrvlSeqNumContextCollection.Find(context => context.EndDate != report.DDCTNS301) != null) ||
            (report.RPProjPoolSeqNumContextCollection != null && report.RPProjPoolSeqNumContextCollection.Find(context => context.EndDate != report.DDCTNS301) != null) ||
            (report.RPForestManInvSeqNumContextCollection != null && report.RPForestManInvSeqNumContextCollection.Find(context => context.EndDate != report.DDCTNS301) != null) ||
            (report.RPGiftDonSeqNumContextCollection != null && report.RPGiftDonSeqNumContextCollection.Find(context => context.EndDate != report.DDCTNS301) != null) ||
            (report.RPIntDedSeqNumContextCollection != null && report.RPIntDedSeqNumContextCollection.Find(context => context.EndDate != report.DDCTNS301) != null) ||
            (report.RPDivDedSeqNumContextCollection != null && report.RPDivDedSeqNumContextCollection.Find(context => context.EndDate != report.DDCTNS301) != null) ||
            (report.RPOTHWRSeqNumContextCollection != null && report.RPOTHWRSeqNumContextCollection.Find(context => context.EndDate != report.DDCTNS301) != null) ||
            (report.RPClthngSeqNumContextCollection != null && report.RPClthngSeqNumContextCollection.Find(context => context.EndDate != report.DDCTNS301) != null) ||
            (report.RPOthDedSeqNumContextCollection != null && report.RPOthDedSeqNumContextCollection.Find(context => context.EndDate != report.DDCTNS301) != null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500033",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. Period end date must be the same across all duration contexts.",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000154" } },
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.DDCTNS.000154

        #region VR.ATO.DDCTNS.000155

        /*  VR.ATO.DDCTNS.000155
        At least one deduction detail must be provided

        Legacy Rule Format:
        (^DDCTNS200 = NULL AND ^DDCTNS310 = NULL AND ^DDCTNS311 = NULL AND ^DDCTNS312 = NULL AND ^DDCTNS417 = NULL AND
        Count(^RP.{CarSeqNum}) = 0 AND Count(^RP.{EduSeqNum}) = 0 AND Count(^RP.{MangTaxAffSeqNum}) = 0 AND Count(^RP.{SCSeqNum}) = 0 AND Count(^RP.{TrvlSeqNum}) = 0 AND Count(^RP.{ProjPoolSeqNum}) = 0 AND Count(^RP.{ForestManInvSeqNum}) = 0 AND Count(^RP.{GiftDonSeqNum}) = 0 AND Count(^RP.{IntDedSeqNum}) = 0 AND Count(^RP.{DivDedSeqNum}) = 0 AND Count(^RP.{OTHWRSeqNum}) = 0 AND Count(^RP.{ClthngSeqNum}) = 0)

        Technical Business Rule Format:
        (^DDCTNS200 = NULL AND ^DDCTNS310 = NULL AND ^DDCTNS311 = NULL AND ^DDCTNS312 = NULL AND ^DDCTNS417 = NULL AND
        Count(^RP.{CarSeqNum}) = 0 AND Count(^RP.{EduSeqNum}) = 0 AND Count(^RP.{MangTaxAffSeqNum}) = 0 AND Count(^RP.{SCSeqNum}) = 0 AND Count(^RP.{TrvlSeqNum}) = 0 AND Count(^RP.{ProjPoolSeqNum}) = 0 AND Count(^RP.{ForestManInvSeqNum}) = 0 AND Count(^RP.{GiftDonSeqNum}) = 0 AND Count(^RP.{IntDedSeqNum}) = 0 AND Count(^RP.{DivDedSeqNum}) = 0 AND Count(^RP.{OTHWRSeqNum}) = 0 AND Count(^RP.{ClthngSeqNum}) = 0)

        Data Elements:
        ^DDCTNS200 = RP:IncomeTax.Deduction.EducationCircumstances.Code
        ^DDCTNS310 = RP:IncomeTax.Deduction.LowValuePoolFinancialInvestments.Amount
        ^DDCTNS311 = RP:IncomeTax.Deduction.LowValuePoolRentalProperties.Amount
        ^DDCTNS312 = RP:IncomeTax.Deduction.LowValuePoolOther.Amount
        ^DDCTNS417 = RP:IncomeTax.Deduction.Election.Amount
        */
        protected void VRATODDCTNS000155(List<ProcessMessageDocument> response, DDCTNS2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = ((report.DDCTNS200 == null) && (report.DDCTNS310 == null) && (report.DDCTNS311 == null) && (report.DDCTNS312 == null) && (report.DDCTNS417 == null) && (Count(report.RPCarSeqNumContextCollection) == 0) && (Count(report.RPEduSeqNumContextCollection) == 0) && (Count(report.RPMangTaxAffSeqNumContextCollection) == 0) && (Count(report.RPSCSeqNumContextCollection) == 0) && (Count(report.RPTrvlSeqNumContextCollection) == 0) && (Count(report.RPProjPoolSeqNumContextCollection) == 0) && (Count(report.RPForestManInvSeqNumContextCollection) == 0) && (Count(report.RPGiftDonSeqNumContextCollection) == 0) && (Count(report.RPIntDedSeqNumContextCollection) == 0) && (Count(report.RPDivDedSeqNumContextCollection) == 0) && (Count(report.RPOTHWRSeqNumContextCollection) == 0) && (Count(report.RPClthngSeqNumContextCollection) == 0) && (Count(report.RPOthDedSeqNumContextCollection) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.DDCTNS.000155",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"At least one deduction detail must be provided",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.DDCTNS.000155" } },
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.DDCTNS.000155

        #endregion Manual Rules - Rules marked for manual coding
    }
}
